@@ -7,8 +7,6 @@
 Aggregate top level objects from all Scapy modules.
 """
 
-# flake8: noqa: F403
-
 from scapy.base_classes import *
 from scapy.config import *
 from scapy.dadict import *
@@ -16,6 +14,7 @@ from scapy.data import *
 from scapy.error import *
 from scapy.themes import *
 from scapy.arch import *
+from scapy.interfaces import *
 
 from scapy.plist import *
 from scapy.fields import *
