@@ -6,241 +6,166 @@
 
 """
 Automata with states, transitions and actions.
+
+TODO:
+    - add documentation for ioevent, as_supersocket...
 """
 
-from __future__ import absolute_import
-import types
+import ctypes
 import itertools
-import time
+import logging
 import os
+import random
 import sys
-import traceback
-from select import select
-from collections import deque
 import threading
+import time
+import traceback
+import types
+
+import select
+from collections import deque
+
 from scapy.config import conf
 from scapy.utils import do_graph
-from scapy.error import log_interactive, warning
+from scapy.error import log_runtime, warning
 from scapy.plist import PacketList
 from scapy.data import MTU
 from scapy.supersocket import SuperSocket
 from scapy.consts import WINDOWS
 import scapy.modules.six as six
 
-if WINDOWS:
-    from scapy.error import Scapy_Exception
-    recv_error = Scapy_Exception
-else:
-    recv_error = ()
-
-""" In Windows, select.select is not available for custom objects. Here's the implementation of scapy to re-create this functionality  # noqa: E501
-# Passive way: using no-ressources locks
-               +---------+             +---------------+      +-------------------------+  # noqa: E501
-               |  Start  +------------->Select_objects +----->+Linux: call select.select|  # noqa: E501
-               +---------+             |(select.select)|      +-------------------------+  # noqa: E501
-                                       +-------+-------+
-                                               |
-                                          +----v----+               +--------+
-                                          | Windows |               |Time Out+----------------------------------+  # noqa: E501
-                                          +----+----+               +----+---+                                  |  # noqa: E501
-                                               |                         ^                                      |  # noqa: E501
-      Event                                    |                         |                                      |  # noqa: E501
-        +                                      |                         |                                      |  # noqa: E501
-        |                              +-------v-------+                 |                                      |  # noqa: E501
-        |                       +------+Selectable Sel.+-----+-----------------+-----------+                    |  # noqa: E501
-        |                       |      +-------+-------+     |           |     |           v              +-----v-----+  # noqa: E501
-+-------v----------+            |              |             |           |     |        Passive lock<-----+release_all<------+  # noqa: E501
-|Data added to list|       +----v-----+  +-----v-----+  +----v-----+     v     v            +             +-----------+      |  # noqa: E501
-+--------+---------+       |Selectable|  |Selectable |  |Selectable|   ............         |                                |  # noqa: E501
-         |                 +----+-----+  +-----------+  +----------+                        |                                |  # noqa: E501
-         |                      v                                                           |                                |  # noqa: E501
-         v                 +----+------+   +------------------+               +-------------v-------------------+            |  # noqa: E501
-   +-----+------+          |wait_return+-->+  check_recv:     |               |                                 |            |  # noqa: E501
-   |call_release|          +----+------+   |If data is in list|               |  END state: selectable returned |        +---+--------+  # noqa: E501
-   +-----+--------              v          +-------+----------+               |                                 |        | exit door  |  # noqa: E501
-         |                    else                 |                          +---------------------------------+        +---+--------+  # noqa: E501
-         |                      +                  |                                                                         |  # noqa: E501
-         |                 +----v-------+          |                                                                         |  # noqa: E501
-         +--------->free -->Passive lock|          |                                                                         |  # noqa: E501
-                           +----+-------+          |                                                                         |  # noqa: E501
-                                |                  |                                                                         |  # noqa: E501
-                                |                  v                                                                         |  # noqa: E501
-                                +------------------Selectable-Selector-is-advertised-that-the-selectable-is-readable---------+
-"""
-
 
 class SelectableObject(object):
-    """DEV: to implement one of those, you need to add 2 things to your object:
-    - add "check_recv" function
-    - call "self.call_release" once you are ready to be read
+    if WINDOWS:
+        def __init__(self):
+            self._fd = ctypes.windll.kernel32.CreateEventA(
+                None, 0, 0,
+                "SelectableObject %s" % random.random()
+            )
 
-    You can set the __selectable_force_select__ to True in the class, if you want to  # noqa: E501
-    force the handler to use fileno(). This may only be usable on sockets created using  # noqa: E501
-    the builtin socket API."""
-    __selectable_force_select__ = False
+        def call_release(self):
+            if ctypes.windll.kernel32.PulseEvent(
+                    ctypes.c_void_p(self._fd)) == 0:
+                warning(ctypes.FormatError())
 
-    def __init__(self):
-        self.hooks = []
+        def _close_fd(self):
+            if self._fd and ctypes.windll.kernel32.CloseHandle(
+                    ctypes.c_void_p(self._fd)) == 0:
+                warning(ctypes.FormatError())
+                self._fd = None
+
+        def __del__(self):
+            if hasattr(self, "_fd"):
+                self._close_fd()
+    else:
+        def call_release(self):
+            pass
+
+        def close(self):
+            pass
 
     def check_recv(self):
-        """DEV: will be called only once (at beginning) to check if the object is ready."""  # noqa: E501
-        raise OSError("This method must be overwritten.")
-
-    def _wait_non_ressources(self, callback):
-        """This get started as a thread, and waits for the data lock to be freed then advertise itself to the SelectableSelector using the callback"""  # noqa: E501
-        self.trigger = threading.Lock()
-        self.was_ended = False
-        self.trigger.acquire()
-        self.trigger.acquire()
-        if not self.was_ended:
-            callback(self)
-
-    def wait_return(self, callback):
-        """Entry point of SelectableObject: register the callback"""
-        if self.check_recv():
-            return callback(self)
-        _t = threading.Thread(target=self._wait_non_ressources, args=(callback,))  # noqa: E501
-        _t.setDaemon(True)
-        _t.start()
-
-    def register_hook(self, hook):
-        """DEV: When call_release() will be called, the hook will also"""
-        self.hooks.append(hook)
-
-    def call_release(self, arborted=False):
-        """DEV: Must be call when the object becomes ready to read.
-           Relesases the lock of _wait_non_ressources"""
-        self.was_ended = arborted
-        try:
-            self.trigger.release()
-        except (threading.ThreadError, AttributeError):
-            pass
-        # Trigger hooks
-        for hook in self.hooks:
-            hook()
-
-
-class SelectableSelector(object):
-    """
-    Select SelectableObject objects.
-
-    inputs: objects to process
-    remain: timeout. If 0, return [].
-    customTypes: types of the objects that have the check_recv function.
-    """
-
-    def _release_all(self):
-        """Releases all locks to kill all threads"""
-        for i in self.inputs:
-            i.call_release(True)
-        self.available_lock.release()
-
-    def _timeout_thread(self, remain):
-        """Timeout before releasing every thing, if nothing was returned"""
-        time.sleep(remain)
-        if not self._ended:
-            self._ended = True
-            self._release_all()
-
-    def _exit_door(self, _input):
-        """This function is passed to each SelectableObject as a callback
-        The SelectableObjects have to call it once there are ready"""
-        self.results.append(_input)
-        if self._ended:
-            return
-        self._ended = True
-        self._release_all()
-
-    def __init__(self, inputs, remain):
-        self.results = []
-        self.inputs = list(inputs)
-        self.remain = remain
-        self.available_lock = threading.Lock()
-        self.available_lock.acquire()
-        self._ended = False
-
-    def process(self):
-        """Entry point of SelectableSelector"""
-        if WINDOWS:
-            select_inputs = []
-            for i in self.inputs:
-                if not isinstance(i, SelectableObject):
-                    warning("Unknown ignored object type: %s", type(i))
-                elif i.__selectable_force_select__:
-                    # Then use select.select
-                    select_inputs.append(i)
-                elif not self.remain and i.check_recv():
-                    self.results.append(i)
-                elif self.remain:
-                    i.wait_return(self._exit_door)
-            if select_inputs:
-                # Use default select function
-                self.results.extend(select(select_inputs, [], [], self.remain)[0])  # noqa: E501
-            if not self.remain:
-                return self.results
-
-            threading.Thread(target=self._timeout_thread, args=(self.remain,)).start()  # noqa: E501
-            if not self._ended:
-                self.available_lock.acquire()
-            return self.results
-        else:
-            r, _, _ = select(self.inputs, [], [], self.remain)
-            return r
+        return False
 
 
 def select_objects(inputs, remain):
     """
     Select SelectableObject objects. Same than:
-        select.select([inputs], [], [], remain)
+    ``select.select(inputs, [], [], remain)``
     But also works on Windows, only on SelectableObject.
 
-    inputs: objects to process
-    remain: timeout. If 0, return [].
+    :param inputs: objects to process
+    :param remain: timeout. If 0, return [].
     """
-    handler = SelectableSelector(inputs, remain)
-    return handler.process()
+    if not WINDOWS:
+        return select.select(inputs, [], [], remain)[0]
+    natives = []
+    events = []
+    results = []
+    for i in list(inputs):
+        if getattr(i, "__selectable_force_select__", False):
+            natives.append(i)
+        elif isinstance(i, SelectableObject):
+            if i.check_recv():
+                results.append(i)
+            else:
+                events.append(i)
+        else:
+            raise TypeError(
+                "Invalid type: %s (must extend SelectableObject)"
+            )
+    if natives:
+        results.extend(select.select(natives, [], [], remain)[0])
+    if events:
+        remainms = int((remain or 0) * 1000)
+        if len(events) == 1:
+            res = ctypes.windll.kernel32.WaitForSingleObject(
+                ctypes.c_void_p(events[0].fileno()),
+                remainms
+            )
+        else:
+            res = ctypes.windll.kernel32.WaitForMultipleObjects(
+                len(events),
+                (ctypes.c_void_p * len(events))(
+                    *[x.fileno() for x in events]
+                ),
+                False,
+                remainms
+            )
+        if res != 0xFFFFFFFF and res != 0x00000102:  # Failed or Timeout
+            results.append(events[res])
+    return results
 
 
 class ObjectPipe(SelectableObject):
-    read_allowed_exceptions = ()
-
     def __init__(self):
-        self.closed = False
-        self.rd, self.wr = os.pipe()
-        self.queue = deque()
+        self._closed = False
+        self.__rd, self.__wr = os.pipe()
+        self.__queue = deque()
         SelectableObject.__init__(self)
 
     def fileno(self):
-        return self.rd
-
-    def check_recv(self):
-        return len(self.queue) > 0
+        if WINDOWS:
+            return self._fd
+        else:
+            return self.__rd
 
     def send(self, obj):
-        self.queue.append(obj)
-        os.write(self.wr, b"X")
+        self.__queue.append(obj)
+        os.write(self.__wr, b"X")
         self.call_release()
 
     def write(self, obj):
         self.send(obj)
 
+    def flush(self):
+        pass
+
+    def check_recv(self):
+        return bool(self.__queue)
+
     def recv(self, n=0):
-        if self.closed:
+        if self._closed:
             if self.check_recv():
-                return self.queue.popleft()
+                return self.__queue.popleft()
             return None
-        os.read(self.rd, 1)
-        return self.queue.popleft()
+        os.read(self.__rd, 1)
+        return self.__queue.popleft()
 
     def read(self, n=0):
         return self.recv(n)
 
     def close(self):
-        if not self.closed:
-            self.closed = True
-            os.close(self.rd)
-            os.close(self.wr)
-            self.queue.clear()
+        if not self._closed:
+            self._closed = True
+            os.close(self.__rd)
+            os.close(self.__wr)
+            self.__queue.clear()
+            if WINDOWS:
+                self._close_fd()
+
+    def __del__(self):
+        self.close()
 
     @staticmethod
     def select(sockets, remain=conf.recv_poll_rate):
@@ -251,7 +176,7 @@ class ObjectPipe(SelectableObject):
                 results.append(s)
         if results:
             return results, None
-        return select_objects(sockets, remain), None
+        return select_objects(sockets, remain)
 
 
 class Message:
@@ -307,6 +232,7 @@ class ATMT:
             self.state = state_func.atmt_state
             self.initial = state_func.atmt_initial
             self.error = state_func.atmt_error
+            self.stop = state_func.atmt_stop
             self.final = state_func.atmt_final
             Exception.__init__(self, "Request state [%s]" % self.state)
             self.automaton = automaton
@@ -326,12 +252,13 @@ class ATMT:
             return "NewStateRequested(%s)" % self.state
 
     @staticmethod
-    def state(initial=0, final=0, error=0):
+    def state(initial=0, final=0, stop=0, error=0):
         def deco(f, initial=initial, final=final):
             f.atmt_type = ATMT.STATE
             f.atmt_state = f.__name__
             f.atmt_initial = initial
             f.atmt_final = final
+            f.atmt_stop = stop
             f.atmt_error = error
 
             def state_wrapper(self, *args, **kargs):
@@ -342,6 +269,7 @@ class ATMT:
             state_wrapper.atmt_state = f.__name__
             state_wrapper.atmt_initial = initial
             state_wrapper.atmt_final = final
+            state_wrapper.atmt_stop = stop
             state_wrapper.atmt_error = error
             state_wrapper.atmt_origfunc = f
             return state_wrapper
@@ -405,6 +333,7 @@ class _ATMT_Command:
     NEXT = "NEXT"
     FREEZE = "FREEZE"
     STOP = "STOP"
+    FORCESTOP = "FORCESTOP"
     END = "END"
     EXCEPTION = "EXCEPTION"
     SINGLESTEP = "SINGLESTEP"
@@ -417,29 +346,27 @@ class _ATMT_Command:
 
 class _ATMT_supersocket(SuperSocket, SelectableObject):
     def __init__(self, name, ioevent, automaton, proto, *args, **kargs):
-        SelectableObject.__init__(self)
         self.name = name
         self.ioevent = ioevent
         self.proto = proto
         # write, read
         self.spa, self.spb = ObjectPipe(), ObjectPipe()
-        # Register recv hook
-        self.spb.register_hook(self.call_release)
         kargs["external_fd"] = {ioevent: (self.spa, self.spb)}
+        kargs["is_atmt_socket"] = True
         self.atmt = automaton(*args, **kargs)
         self.atmt.runbg()
 
-    def fileno(self):
-        return self.spb.fileno()
-
     def send(self, s):
         if not isinstance(s, bytes):
             s = bytes(s)
-        return self.spa.send(s)
+        self.spa.send(s)
 
     def check_recv(self):
         return self.spb.check_recv()
 
+    def fileno(self):
+        return self.spb.fileno()
+
     def recv(self, n=MTU):
         r = self.spb.recv(n)
         if self.proto is not None:
@@ -447,11 +374,15 @@ class _ATMT_supersocket(SuperSocket, SelectableObject):
         return r
 
     def close(self):
-        pass
+        if not self.closed:
+            self.atmt.stop()
+            self.spa.close()
+            self.spb.close()
+            self.closed = True
 
     @staticmethod
     def select(sockets, remain=conf.recv_poll_rate):
-        return select_objects(sockets, remain), None
+        return select_objects(sockets, remain)
 
 
 class _ATMT_to_supersocket:
@@ -478,6 +409,7 @@ class Automaton_metaclass(type):
         cls.timeout = {}
         cls.actions = {}
         cls.initial_states = []
+        cls.stop_states = []
         cls.ionames = []
         cls.iosupersockets = []
 
@@ -503,6 +435,8 @@ class Automaton_metaclass(type):
                 cls.timeout[s] = []
                 if m.atmt_initial:
                     cls.initial_states.append(m)
+                if m.atmt_stop:
+                    cls.stop_states.append(m)
             elif m.atmt_type in [ATMT.CONDITION, ATMT.RECV, ATMT.TIMEOUT, ATMT.IOEVENT]:  # noqa: E501
                 cls.actions[m.atmt_condname] = []
 
@@ -548,6 +482,8 @@ class Automaton_metaclass(type):
                 se += '\t"%s" [ style=filled, fillcolor=green, shape=octagon ];\n' % st.atmt_state  # noqa: E501
             elif st.atmt_error:
                 se += '\t"%s" [ style=filled, fillcolor=red, shape=octagon ];\n' % st.atmt_state  # noqa: E501
+            elif st.atmt_stop:
+                se += '\t"%s" [ style=filled, fillcolor=orange, shape=box, root=true ];\n' % st.atmt_state  # noqa: E501
         s += se
 
         for st in six.itervalues(self.states):
@@ -586,6 +522,8 @@ class Automaton_metaclass(type):
 class Automaton(six.with_metaclass(Automaton_metaclass)):
     def parse_args(self, debug=0, store=1, **kargs):
         self.debug_level = debug
+        if debug:
+            conf.logLevel = logging.DEBUG
         self.socket_kargs = kargs
         self.store_packets = store
 
@@ -620,7 +558,6 @@ class Automaton(six.with_metaclass(Automaton_metaclass)):
             return os.read(self.rd, n)
 
         def write(self, msg):
-            self.call_release()
             if isinstance(self.wr, ObjectPipe):
                 self.wr.send(msg)
                 return
@@ -653,8 +590,7 @@ class Automaton(six.with_metaclass(Automaton_metaclass)):
             return self.recv(n)
 
         def send(self, msg):
-            self.wr.send(msg)
-            return self.call_release()
+            return self.wr.send(msg)
 
         def write(self, msg):
             return self.send(msg)
@@ -694,7 +630,7 @@ class Automaton(six.with_metaclass(Automaton_metaclass)):
     # Services
     def debug(self, lvl, msg):
         if self.debug_level >= lvl:
-            log_interactive.debug(msg)
+            log_runtime.debug(msg)
 
     def send(self, pkt):
         if self.state.state in self.interception_points:
@@ -725,6 +661,7 @@ class Automaton(six.with_metaclass(Automaton_metaclass)):
         external_fd = kargs.pop("external_fd", {})
         self.send_sock_class = kargs.pop("ll", conf.L3socket)
         self.recv_sock_class = kargs.pop("recvsock", conf.L2listen)
+        self.is_atmt_socket = kargs.pop("is_atmt_socket", False)
         self.started = threading.Lock()
         self.threadid = None
         self.breakpointed = None
@@ -794,7 +731,12 @@ class Automaton(six.with_metaclass(Automaton_metaclass)):
 
     def _do_start(self, *args, **kargs):
         ready = threading.Event()
-        _t = threading.Thread(target=self._do_control, args=(ready,) + (args), kwargs=kargs)  # noqa: E501
+        _t = threading.Thread(
+            target=self._do_control,
+            args=(ready,) + (args),
+            kwargs=kargs,
+            name="scapy.automaton _do_start"
+        )
         _t.setDaemon(True)
         _t.start()
         ready.wait()
@@ -831,6 +773,14 @@ class Automaton(six.with_metaclass(Automaton_metaclass)):
                     elif c.type == _ATMT_Command.FREEZE:
                         continue
                     elif c.type == _ATMT_Command.STOP:
+                        if self.stop_states:
+                            # There is a stop state
+                            self.state = self.stop_states[0](self)
+                            iterator = self._do_iter()
+                        else:
+                            # Act as FORCESTOP
+                            break
+                    elif c.type == _ATMT_Command.FORCESTOP:
                         break
                     while True:
                         state = next(iterator)
@@ -880,16 +830,20 @@ class Automaton(six.with_metaclass(Automaton_metaclass)):
                 elif not isinstance(state_output, list):
                     state_output = state_output,
 
-                # Then check immediate conditions
-                for cond in self.conditions[self.state.state]:
-                    self._run_condition(cond, *state_output)
+                # If there are commandMessage, we should skip immediate
+                # conditions.
+                if not select_objects([self.cmdin], 0):
+                    # Then check immediate conditions
+                    for cond in self.conditions[self.state.state]:
+                        self._run_condition(cond, *state_output)
 
-                # If still there and no conditions left, we are stuck!
-                if (len(self.recv_conditions[self.state.state]) == 0 and
-                    len(self.ioevents[self.state.state]) == 0 and
-                        len(self.timeout[self.state.state]) == 1):
-                    raise self.Stuck("stuck in [%s]" % self.state.state,
-                                     state=self.state.state, result=state_output)  # noqa: E501
+                    # If still there and no conditions left, we are stuck!
+                    if (len(self.recv_conditions[self.state.state]) == 0 and
+                        len(self.ioevents[self.state.state]) == 0 and
+                            len(self.timeout[self.state.state]) == 1):
+                        raise self.Stuck("stuck in [%s]" % self.state.state,
+                                         state=self.state.state,
+                                         result=state_output)
 
                 # Finally listen and pay attention to timeouts
                 expirations = iter(self.timeout[self.state.state])
@@ -920,18 +874,14 @@ class Automaton(six.with_metaclass(Automaton_metaclass)):
                         if fd == self.cmdin:
                             yield self.CommandMessage("Received command message")  # noqa: E501
                         elif fd == self.listen_sock:
-                            try:
-                                pkt = self.listen_sock.recv(MTU)
-                            except recv_error:
-                                pass
-                            else:
-                                if pkt is not None:
-                                    if self.master_filter(pkt):
-                                        self.debug(3, "RECVD: %s" % pkt.summary())  # noqa: E501
-                                        for rcvcond in self.recv_conditions[self.state.state]:  # noqa: E501
-                                            self._run_condition(rcvcond, pkt, *state_output)  # noqa: E501
-                                    else:
-                                        self.debug(4, "FILTR: %s" % pkt.summary())  # noqa: E501
+                            pkt = self.listen_sock.recv(MTU)
+                            if pkt is not None:
+                                if self.master_filter(pkt):
+                                    self.debug(3, "RECVD: %s" % pkt.summary())  # noqa: E501
+                                    for rcvcond in self.recv_conditions[self.state.state]:  # noqa: E501
+                                        self._run_condition(rcvcond, pkt, *state_output)  # noqa: E501
+                                else:
+                                    self.debug(4, "FILTR: %s" % pkt.summary())  # noqa: E501
                         else:
                             self.debug(3, "IOEVENT on %s" % fd.ioname)
                             for ioevt in self.ioevents[self.state.state]:
@@ -943,6 +893,12 @@ class Automaton(six.with_metaclass(Automaton_metaclass)):
                 self.state = state_req
                 yield state_req
 
+    def __repr__(self):
+        return "<Automaton %s [%s]>" % (
+            self.__class__.__name__,
+            ["HALTED", "RUNNING"][self.started.locked()]
+        )
+
     # Public API
     def add_interception_points(self, *ipts):
         for ipt in ipts:
@@ -1000,8 +956,7 @@ class Automaton(six.with_metaclass(Automaton_metaclass)):
         return self.run(resume=Message(type=_ATMT_Command.NEXT))
     __next__ = next
 
-    def stop(self):
-        self.cmdin.send(Message(type=_ATMT_Command.STOP))
+    def _flush_inout(self):
         with self.started:
             # Flush command pipes
             while True:
@@ -1011,6 +966,14 @@ class Automaton(six.with_metaclass(Automaton_metaclass)):
                 for fd in r:
                     fd.recv()
 
+    def stop(self):
+        self.cmdin.send(Message(type=_ATMT_Command.STOP))
+        self._flush_inout()
+
+    def forcestop(self):
+        self.cmdin.send(Message(type=_ATMT_Command.FORCESTOP))
+        self._flush_inout()
+
     def restart(self, *args, **kargs):
         self.stop()
         self.start(*args, **kargs)
