@@ -10,19 +10,20 @@ DNS: Domain Name System.
 from __future__ import absolute_import
 import struct
 import time
+import warnings
 
 from scapy.config import conf
 from scapy.packet import Packet, bind_layers, NoPayload
 from scapy.fields import BitEnumField, BitField, ByteEnumField, ByteField, \
-    ConditionalField, FieldLenField, FlagsField, IntField, \
-    PacketListField, ShortEnumField, ShortField, StrField, StrFixedLenField, \
+    ConditionalField, Field, FieldLenField, FlagsField, IntField, \
+    PacketListField, ShortEnumField, ShortField, StrField, \
     StrLenField, MultipleTypeField, UTCTimeField
 from scapy.compat import orb, raw, chb, bytes_encode
 from scapy.ansmachine import AnsweringMachine
 from scapy.sendrecv import sr1
 from scapy.layers.inet import IP, DestIPField, IPField, UDP, TCP
 from scapy.layers.inet6 import DestIP6Field, IP6Field
-from scapy.error import warning, Scapy_Exception
+from scapy.error import log_runtime, warning, Scapy_Exception
 import scapy.modules.six as six
 from scapy.modules.six.moves import range
 
@@ -55,8 +56,9 @@ def dns_get_str(s, pointer=0, pkt=None, _fullpacket=False):
     bytes_left = None
     while True:
         if abs(pointer) >= max_length:
-            warning("DNS RR prematured end (ofs=%i, len=%i)" % (pointer,
-                                                                len(s)))
+            log_runtime.info(
+                "DNS RR prematured end (ofs=%i, len=%i)", pointer, len(s)
+            )
             break
         cur = orb(s[pointer])  # get pointer value
         pointer += 1  # make pointer go forward
@@ -66,7 +68,9 @@ def dns_get_str(s, pointer=0, pkt=None, _fullpacket=False):
                 # as pointer will follow the jump token
                 after_pointer = pointer + 1
             if pointer >= max_length:
-                warning("DNS incomplete jump token at (ofs=%i)" % pointer)
+                log_runtime.info(
+                    "DNS incomplete jump token at (ofs=%i)", pointer
+                )
                 break
             # Follow the pointer
             pointer = ((cur & ~0xc0) << 8) + orb(s[pointer]) - 12
@@ -83,7 +87,7 @@ def dns_get_str(s, pointer=0, pkt=None, _fullpacket=False):
                     _fullpacket = True
                 else:
                     # No -> abort
-                    raise Scapy_Exception("DNS message can't be compressed" +
+                    raise Scapy_Exception("DNS message can't be compressed " +
                                           "at this point!")
             processed_pointers.append(pointer)
             continue
@@ -113,7 +117,8 @@ def dns_encode(x, check_built=False):
         return b"\x00"
 
     if check_built and b"." not in x and (
-        orb(x[-1]) == 0 or (orb(x[-2]) & 0xc0) == 0xc0
+        (x and orb(x[-1]) == 0) or
+        (len(x) >= 2 and (orb(x[-2]) & 0xc0) == 0xc0)
     ):
         # The value has already been processed. Do not process it again
         return x
@@ -127,7 +132,10 @@ def dns_encode(x, check_built=False):
 
 def DNSgetstr(*args, **kwargs):
     """Legacy function. Deprecated"""
-    warning("DNSgetstr deprecated. Use dns_get_str instead")
+    warnings.warn(
+        "DNSgetstr is deprecated. Use dns_get_str instead.",
+        DeprecationWarning
+    )
     return dns_get_str(*args, **kwargs)
 
 
@@ -138,6 +146,7 @@ def dns_compress(pkt):
         raise Scapy_Exception("Can only compress DNS layers")
     pkt = pkt.copy()
     dns_pkt = pkt.getlayer(DNS)
+    dns_pkt.clear_cache()
     build_pkt = raw(dns_pkt)
 
     def field_gen(dns_pkt):
@@ -163,7 +172,6 @@ def dns_compress(pkt):
         for x in range(1, dat.count(b".")):
             yield dat.split(b".", x)[x]
     data = {}
-    burned_data = 0
     for current, name, dat in field_gen(dns_pkt):
         for part in possible_shortens(dat):
             # Encode the data
@@ -173,18 +181,21 @@ def dns_compress(pkt):
                 # possible pointer for future strings.
                 # We get the index of the encoded data
                 index = build_pkt.index(encoded)
-                index -= burned_data
                 # The following is used to build correctly the pointer
                 fb_index = ((index >> 8) | 0xc0)
                 sb_index = index - (256 * (fb_index - 0xc0))
                 pointer = chb(fb_index) + chb(sb_index)
-                data[part] = [(current, name, pointer)]
+                data[part] = [(current, name, pointer, index + 1)]
             else:
                 # This string already exists, let's mark the current field
                 # with it, so that it gets compressed
                 data[part].append((current, name))
-                # calculate spared space
-                burned_data += len(encoded) - 2
+                _in = data[part][0][3]
+                build_pkt = build_pkt[:_in] + build_pkt[_in:].replace(
+                    encoded,
+                    b"\0\0",
+                    1
+                )
                 break
     # Apply compression rules
     for ck in data:
@@ -229,7 +240,6 @@ class DNSStrField(StrLenField):
     It will also handle DNS decompression.
     (may be StrLenField if a length_from is passed),
     """
-
     def h2i(self, pkt, x):
         if not x:
             return b"."
@@ -244,7 +254,7 @@ class DNSStrField(StrLenField):
     def getfield(self, pkt, s):
         remain = b""
         if self.length_from:
-            remain, s = StrLenField.getfield(self, pkt, s)
+            remain, s = super(DNSStrField, self).getfield(pkt, s)
         # Decode the compressed DNS message
         decoded, _, left = dns_get_str(s, 0, pkt)
         # returns (remaining, decoded)
@@ -261,7 +271,7 @@ class DNSRRCountField(ShortField):
     def _countRR(self, pkt):
         x = getattr(pkt, self.rr)
         i = 0
-        while isinstance(x, DNSRR) or isinstance(x, DNSQR) or isdnssecRR(x):
+        while isinstance(x, (DNSRR, DNSQR)) or isdnssecRR(x):
             x = x.payload
             i += 1
         return i
@@ -313,7 +323,7 @@ class DNSRRField(StrField):
         ret = None
         c = getattr(pkt, self.countfld)
         if c > len(s):
-            warning("wrong value: DNS.%s=%i", self.countfld, c)
+            log_runtime.info("DNS wrong value: DNS.%s=%i", self.countfld, c)
             return s, b""
         while c:
             c -= 1
@@ -353,7 +363,10 @@ class DNSTextField(StrLenField):
         while tmp_s:
             tmp_len = orb(tmp_s[0]) + 1
             if tmp_len > len(tmp_s):
-                warning("DNS RR TXT prematured end of character-string (size=%i, remaining bytes=%i)" % (tmp_len, len(tmp_s)))  # noqa: E501
+                log_runtime.info(
+                    "DNS RR TXT prematured end of character-string "
+                    "(size=%i, remaining bytes=%i)", tmp_len, len(tmp_s)
+                )
             ret_s.append(tmp_s[1:tmp_len])
             tmp_s = tmp_s[tmp_len:]
         return ret_s
@@ -417,7 +430,6 @@ class DNS(Packet):
                 other.qr == 0)
 
     def mysummary(self):
-        type = ["Qry", "Ans"][self.qr]
         name = ""
         if self.qr:
             type = "Ans"
@@ -438,6 +450,28 @@ class DNS(Packet):
         """Return the compressed DNS packet (using `dns_compress()`"""
         return dns_compress(self)
 
+    def pre_dissect(self, s):
+        """
+        Check that a valid DNS over TCP message can be decoded
+        """
+        if isinstance(self.underlayer, TCP):
+
+            # Compute the length of the DNS packet
+            if len(s) >= 2:
+                dns_len = struct.unpack("!H", s[:2])[0]
+            else:
+                message = "Malformed DNS message: too small!"
+                log_runtime.info(message)
+                raise Scapy_Exception(message)
+
+            # Check if the length is valid
+            if dns_len < 14 or len(s) < dns_len:
+                message = "Malformed DNS message: invalid length!"
+                log_runtime.info(message)
+                raise Scapy_Exception(message)
+
+        return s
+
 
 # https://www.iana.org/assignments/dns-parameters/dns-parameters.xhtml#dns-parameters-4
 dnstypes = {
@@ -524,7 +558,7 @@ def bitmap2RRlist(bitmap):
     while bitmap:
 
         if len(bitmap) < 2:
-            warning("bitmap too short (%i)" % len(bitmap))
+            log_runtime.info("bitmap too short (%i)", len(bitmap))
             return
 
         window_block = orb(bitmap[0])  # window number
@@ -532,7 +566,7 @@ def bitmap2RRlist(bitmap):
         bitmap_len = orb(bitmap[1])  # length of the bitmap in bytes
 
         if bitmap_len <= 0 or bitmap_len > 32:
-            warning("bitmap length is no valid (%i)" % bitmap_len)
+            log_runtime.info("bitmap length is no valid (%i)", bitmap_len)
             return
 
         tmp_bitmap = bitmap[2:2 + bitmap_len]
@@ -786,9 +820,9 @@ tsig_algo_sizes = {"HMAC-MD5.SIG-ALG.REG.INT": 16,
                    "hmac-sha1": 20}
 
 
-class TimeSignedField(StrFixedLenField):
+class TimeSignedField(Field[int, bytes]):
     def __init__(self, name, default):
-        StrFixedLenField.__init__(self, name, default, 6)
+        Field.__init__(self, name, default, fmt="6s")
 
     def _convert_seconds(self, packed_seconds):
         """Unpack the internal representation."""
@@ -796,7 +830,7 @@ class TimeSignedField(StrFixedLenField):
         seconds += struct.unpack("!I", packed_seconds[2:])[0]
         return seconds
 
-    def h2i(self, pkt, seconds):
+    def i2m(self, pkt, seconds):
         """Convert the number of seconds since 1-Jan-70 UTC to the packed
            representation."""
 
@@ -808,7 +842,7 @@ class TimeSignedField(StrFixedLenField):
 
         return struct.pack("!HI", tmp_short, tmp_int)
 
-    def i2h(self, pkt, packed_seconds):
+    def m2i(self, pkt, packed_seconds):
         """Convert the internal representation to the number of seconds
            since 1-Jan-70 UTC."""
 
@@ -820,7 +854,7 @@ class TimeSignedField(StrFixedLenField):
     def i2repr(self, pkt, packed_seconds):
         """Convert the internal representation to a nice one using the RFC
            format."""
-        time_struct = time.gmtime(self._convert_seconds(packed_seconds))
+        time_struct = time.gmtime(packed_seconds)
         return time.strftime("%a %b %d %H:%M:%S %Y", time_struct)
 
 
