@@ -8,7 +8,6 @@ SSLv2 handshake fields & logic.
 
 import struct
 
-from scapy.compat import chb
 from scapy.error import log_runtime, warning
 from scapy.utils import randstring
 from scapy.fields import ByteEnumField, ByteField, EnumField, FieldLenField, \
@@ -142,6 +141,8 @@ class _SSLv2CertDataField(StrLenField):
         try:
             certdata = Cert(s[:tmp_len])
         except Exception:
+            # Packets are sometimes wrongly interpreted as SSLv2
+            # (see record.py). We ignore failures silently
             certdata = s[:tmp_len]
         return s[tmp_len:], certdata
 
@@ -296,7 +297,7 @@ class SSLv2ClientMasterKey(_SSLv2Handshake):
             cipher = pkt[1:4]
             cs_val = struct.unpack("!I", b"\x00" + cipher)[0]
             if cs_val not in _tls_cipher_suites_cls:
-                warning("Unknown ciphersuite %d from ClientMasterKey" % cs_val)
+                warning("Unknown cipher suite %d from ClientMasterKey", cs_val)
                 cs_cls = None
             else:
                 cs_cls = _tls_cipher_suites_cls[cs_val]
@@ -337,7 +338,7 @@ class SSLv2ClientMasterKey(_SSLv2Handshake):
             self.keyarglen = len(keyarg)
         keyarglen = struct.pack("!H", self.keyarglen)
 
-        s = (chb(pkt[0]) + cipher +
+        s = (pkt[:1] + cipher +
              clearkeylen + encryptedkeylen + keyarglen +
              clearkey + encryptedkey + keyarg)
         return s + pay
@@ -348,7 +349,7 @@ class SSLv2ClientMasterKey(_SSLv2Handshake):
         s = self.tls_session
         cs_val = self.cipher
         if cs_val not in _tls_cipher_suites_cls:
-            warning("Unknown cipher suite %d from ClientMasterKey" % cs_val)
+            warning("Unknown cipher suite %d from ClientMasterKey", cs_val)
             cs_cls = None
         else:
             cs_cls = _tls_cipher_suites_cls[cs_val]
